--
-- SingleDiscSpreader
--
-- @package 	fs19.apv_es_100_m1.specializations.singleDiscSpreader
-- @auhtor 		IceBlade <www.cg-production.com>
-- @copyright 	2020 CG-Production
-- @license    	www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version		1.0.0.0
-- @history		<1.0.0.0> <14.04.2020> creation
-- @description This script is used to define missing functions from specs when switching vehicleTypeConfiguration
--

SingleDiscSpreader = {};
SingleDiscSpreader.MOD_NAME = g_currentModName;

AttacherJoints.registerJointType("singleDiscSpreader");

function SingleDiscSpreader.prerequisitesPresent(specializations)
    return true;
end;

function SingleDiscSpreader.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "onAttachSpreader", SingleDiscSpreader.onAttachSpreader)
    SpecializationUtil.registerFunction(vehicleType, "onDetachSpreader", SingleDiscSpreader.onDetachSpreader)
    SpecializationUtil.registerFunction(vehicleType, "setSuspensionNodeVisibility", SingleDiscSpreader.setSuspensionNodeVisibility)
end;

function SingleDiscSpreader.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onPreLoad", SingleDiscSpreader);
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", SingleDiscSpreader);
end;

function SingleDiscSpreader:onPreLoad(savegame)
    local i = 0;
    local key = "vehicle.singleDiscSpreader";
    while true do
        local loopKey = (key .. ".vehicleTypeChange.function(%d)"):format(i);

        if not hasXMLProperty(self.xmlFile, loopKey) then
            break
        end

        local func = getXMLString(self.xmlFile, loopKey .. "#name");

        if func ~= nil then
            if self[func] == nil then
                self[func] = function(workArea, dt) return 0, 0; end;
            end;
        end;

        i = i + 1;
    end;
end;

function SingleDiscSpreader:onLoad(savegame)
    self.spec_singleDiscSpreader = self[("spec_%s.singleDiscSpreader"):format(SingleDiscSpreader.MOD_NAME)]
    local spec = self.spec_singleDiscSpreader;

    local key = "vehicle.singleDiscSpreader";

    spec.holderPlates = {};
    spec.holderPlates.node = {};
    spec.holderPlates.node.main = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".holderPlates#node"), self.i3dMappings);
    spec.holderPlates.node.lower = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, key .. ".holderPlates#lowerNode"), self.i3dMappings);

    if spec.holderPlates.node.lower ~= nil then
        spec.holderPlates.node.lowerDefaults = {};

        spec.holderPlates.node.lowerDefaults.rot = {};
        spec.holderPlates.node.lowerDefaults.rot.x, spec.holderPlates.node.lowerDefaults.rot.y, spec.holderPlates.node.lowerDefaults.rot.z = getRotation(spec.holderPlates.node.lower);

        spec.holderPlates.node.lowerDefaults.trans = {};
        spec.holderPlates.node.lowerDefaults.trans.x, spec.holderPlates.node.lowerDefaults.trans.y, spec.holderPlates.node.lowerDefaults.trans.z = getTranslation(spec.holderPlates.node.lower);
    end;

    -- suspension nodes
    spec.suspensionNodes = {};

    local i = 0;
    while true do
        local loopKey = (key .. ".suspensionNodes.node(%d)"):format(i);

        if not hasXMLProperty(self.xmlFile, loopKey) then
            break
        end

        local node = I3DUtil.indexToObject(self.components, getXMLString(self.xmlFile, loopKey .. "#node"), self.i3dMappings);

        table.insert(spec.suspensionNodes, node);

        i = i + 1;
    end;
end;

function SingleDiscSpreader:onAttachSpreader(vehicle, parameters, event)
    local spec = self.spec_singleDiscSpreader;

    -- suspension
    spec:setSuspensionNodeVisibility(false);

    -- holder plate main
    if spec.holderPlates.node.main ~= nil then
        if parameters.holderPlates.show == nil then
            parameters.holderPlates.show = true;
        end;

        setVisibility(spec.holderPlates.node.main, parameters.holderPlates.show);
    end;

    -- holder plate lower
    if spec.holderPlates.node.lower ~= nil then
        if parameters.holderPlates.adjustLowerPlate ~= nil and parameters.holderPlates.adjustLowerPlate == true then
            if parameters.holderPlates.lowerOffset == nil then
                parameters.holderPlates.lowerOffset = 0;
            end;
            setTranslation(spec.holderPlates.node.lower, spec.holderPlates.node.lowerDefaults.trans.x, parameters.holderPlates.lowerOffset, spec.holderPlates.node.lowerDefaults.trans.z);
        end;
    end;
end;

function SingleDiscSpreader:onDetachSpreader(vehicle, parameters, event)
    local spec = self.spec_singleDiscSpreader;

    -- suspension
    spec:setSuspensionNodeVisibility(true);

    -- holder plate main
    if spec.holderPlates.node.main ~= nil then
        setVisibility(spec.holderPlates.node.main, true);
    end;

    -- holder plate lower
    if spec.holderPlates.node.lower ~= nil then
        setTranslation(spec.holderPlates.node.lower, spec.holderPlates.node.lowerDefaults.trans.x, spec.holderPlates.node.lowerDefaults.trans.y, spec.holderPlates.node.lowerDefaults.trans.z);
    end;
end;

function SingleDiscSpreader:setSuspensionNodeVisibility(visibility)
    local spec = self.spec_singleDiscSpreader;

    for _, node in pairs(spec.suspensionNodes) do
        setVisibility(node, visibility)
    end;
end;