--
-- SingleDiscSpreaderAttacher
--
-- @package     fs19.apv_es_100_m1.specializations.singleDiscSpreaderAttacher
-- @auhtor      IceBlade <www.cg-production.com>
-- @copyright   2020 CG-Production
-- @license     www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version     1.0.0.0
-- @history     <1.0.0.0> <16.04.2020> creation
--

SingleDiscSpreaderAttacher = {};
SingleDiscSpreaderAttacher.MOD_DIR = g_currentModDirectory;
SingleDiscSpreaderAttacher.MOD_NAME = g_currentModName;

AttacherJoints.registerJointType("singleDiscSpreader");

function SingleDiscSpreaderAttacher.prerequisitesPresent(specializations)
    return true;
end;

function SingleDiscSpreaderAttacher.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "loadFromXML", SingleDiscSpreaderAttacher.loadFromXML)
    SpecializationUtil.registerFunction(vehicleType, "hasSingleDiscSpreaderAttacher", SingleDiscSpreaderAttacher.hasSingleDiscSpreaderAttacher)
end;

function SingleDiscSpreaderAttacher.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", SingleDiscSpreaderAttacher);
    SpecializationUtil.registerEventListener(vehicleType, "onPostAttachImplement", SingleDiscSpreaderAttacher);
    SpecializationUtil.registerEventListener(vehicleType, "onPreDetachImplement", SingleDiscSpreaderAttacher);
    SpecializationUtil.registerEventListener(vehicleType, "onPostDetachImplement", SingleDiscSpreaderAttacher);
end;

function SingleDiscSpreaderAttacher:onLoad(savegame)
    self.spec_singleDiscSpreaderAttacher = self[("spec_%s.singleDiscSpreaderAttacher"):format(SingleDiscSpreaderAttacher.MOD_NAME)]
    local spec = self.spec_singleDiscSpreaderAttacher;

    self:loadFromXML(self.xmlFile);
end;

function SingleDiscSpreaderAttacher:loadFromXML(xml)
    local spec = self.spec_singleDiscSpreaderAttacher;

    local key = "vehicle.singleDiscSpreaderAttacher";

    spec.attachParameters = {};

    -- holder plates
    spec.attachParameters.holderPlates = {};
    spec.attachParameters.holderPlates.show = false;
    spec.attachParameters.holderPlates.lowerOffset = 0;
    spec.attachParameters.holderPlates.adjustLowerPlate = false;

    if hasXMLProperty(xml, key .. ".holderPlates") then
        spec.attachParameters.holderPlates.show = Utils.getNoNil(getXMLBool(xml, key .. ".holderPlates#show"), false);
        spec.attachParameters.holderPlates.lowerOffset = Utils.getNoNil(getXMLFloat(xml, key .. ".holderPlates#lowerOffset"), 0);
        spec.attachParameters.holderPlates.adjustLowerPlate = Utils.getNoNil(getXMLBool(xml, key .. ".holderPlates#adjustLowerPlate"), false);
    end;

    -- detachPosition
    if hasXMLProperty(xml, key .. ".detachPosition") then
        spec.detachPosition = {};
        spec.detachPosition.trans = {};
        spec.detachPosition.rot = {};

        spec.detachPosition.trans.x, spec.detachPosition.trans.y, spec.detachPosition.trans.z = StringUtil.getVectorFromString(Utils.getNoNil(getXMLString(xml, key .. ".detachPosition#trans"), "0 0 0"));
        spec.detachPosition.rot.x, spec.detachPosition.rot.y, spec.detachPosition.rot.z = StringUtil.getVectorFromString(Utils.getNoNil(getXMLString(xml, key .. ".detachPosition#rot"), "0 0 0"));
    end;
end;

function SingleDiscSpreaderAttacher:onPostAttachImplement(attachable, inputJointDescIndex, jointDescIndex)
    if attachable['onAttachSpreader'] ~= nil then
        local spec = self.spec_singleDiscSpreaderAttacher;
        attachable:onAttachSpreader(self, spec.attachParameters, "attach");
    end;
end;

function SingleDiscSpreaderAttacher:onPreDetachImplement(implement)
    if implement.object['onDetachSpreader'] ~= nil then
        local spec = self.spec_singleDiscSpreaderAttacher;
        implement.object:onDetachSpreader(self, spec.attachParameters, "detach");
    end;
end;

function SingleDiscSpreaderAttacher:onPostDetachImplement(implementIndex)
    local spec = self.spec_singleDiscSpreaderAttacher;

    if self.getObjectFromImplementIndex ~= nil then
        local object = self:getObjectFromImplementIndex(implementIndex);

        if object ~= nil then
            if object.spec_smartAttach ~= nil and object.spec_smartAttach.targetVehicle == nil then
                if spec.detachPosition ~= nil then
                    local x,y,z = localToWorld(self.rootNode, spec.detachPosition.trans.x, spec.detachPosition.trans.y, spec.detachPosition.trans.z);
                    local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, y, z) + 0.2;
                    local component = 1;
                    local xR, yR, zR = getWorldRotation(self.rootNode);

                    removeFromPhysics(object.rootNode);
                    object:setWorldPosition(x, terrainHeight, z, xR - spec.detachPosition.rot.x, yR - spec.detachPosition.rot.y, zR - spec.detachPosition.rot.z, component, true);
                    addToPhysics(object.rootNode);
                end;
            end;
        end;
    end;
end;

function SingleDiscSpreaderAttacher:hasSingleDiscSpreaderAttacher()
    return true;
end;